/* 
Bartomiej Jurek, gr. H1SIDAK
*/

/*
OPIS ALGORYTMU
Algorytm set_intersection suy do wyznaczenia posortowanego przecicia dwch 
zbiorw, ktrych elementy zawarte s w dwch posortowanyc kontenerach. 
Elementy te le w zakresach [first1, last1), [first2, last2). 
Wartoci zwracan jest koniec wyjciowego zbioru.

Dostpne s nastpujce wersje algorytmu:

template <class InputIterator1, class InputIterator2, class OutputIterator>
OutputIterator set_intersection(InputIterator1 first1, InputIterator1 last1,
                                InputIterator2 first2, InputIterator2 last2,
                                OutputIterator result);

template <class InputIterator1, class InputIterator2, class OutputIterator,
          class StrictWeakOrdering>
OutputIterator set_intersection(InputIterator1 first1, InputIterator1 last1,
                                InputIterator2 first2, InputIterator2 last2,
                                OutputIterator result, 
                                StrictWeakOrdering comp);

Pierwsza zakada, e elementy s posortowane standardowym operatorem <. Jeli
tak nie jest i dostarczono wasnego komparatora, ktry zapewnia zachowanie 
relacji sabego porzdku, mona uy wersji drugiej.

set_intersection jest stabilny, czyli elementy w zwrconym kontenerze bd w 
relatywnie tym samym porzdku co w pierwszym argumencie oraz e elementy s
kopiowane raczej z pierwszego ni drugiego kontenera.

Zoono algorytmu jest liniowa.
*/

/*
OPIS PROBLEMU
Zamy, e dysponujemy klas hodowcy psw rasowych (Breeder), ktra zawiera 
list gatunkw psw, ktre s przez niego hodowane.
Dla dwch hodowcw naley wyznaczy te gatunki, ktre s hodowane przez obydwu,
aby mona byo ich uy w reprodukcji.
W tym celu posuymy si algorytmem set_intersection.
*/

#include <algorithm> // set_intersection, for_each
#include <iostream>
#include <string>
#include <vector>

using std::cin;
using std::cout;
using std::endl;
using std::string;

// pomocniczy szablon sucy do wywietlania elementw z kolekcji w jednym wierszu
template <typename T>
void show_collection(const T& col)
{
	for (T::const_iterator i = col.begin(); i != col.end(); ++i)
	{
		cout << *i << ",  ";
	}
	cout << endl;
}
// pomocnicza definicja typu
typedef std::vector<std::string> StringVector;

// klasa hodowcy
class Breeder
{
public:
	Breeder(const char* breeds[], size_t size)
	{
		for (unsigned i = 0; i < size; ++i)
		{
			breeds_.push_back(string(breeds[i]));
		}
	}
	// Zwraca referencj do posortowanych wektorw ras psw
	StringVector& getSortedBreeds() 
	{	
		std::sort(breeds_.begin(),breeds_.end());
		return breeds_;
	}
private:
	StringVector breeds_;
};

/*
Funkcja wyznaczajca rasy psw wsplne dla dwch hodowcw 
*/
StringVector CommonBreeds(Breeder& a, Breeder& b)
{
	StringVector common_breeds;
	// do umieszczania elementw w wyjciowym kontenerze common_breeds uyto
	// iteratora wyjciowego, back_inserter, ktry umieszcza elementy na
	// kocu kontenera
	std::set_intersection(a.getSortedBreeds().begin(),a.getSortedBreeds().end(),
						  b.getSortedBreeds().begin(),b.getSortedBreeds().end(),
						  std::back_inserter(common_breeds));
	return common_breeds;
}

int main()
{
	// tablice ras psw dla dwch hodowcw
	const char* breeds1[] = {"boxer", "terrier","spaniel","mastiff","chart",
							 "shar-pei","bullterier"};

	const char* breeds2[] = {"terrier", "shar-pei","spaniel", "mastiff", 
							 "chihuahua","fila brasileiro","bullmastiff"};
	const unsigned SIZE = 7;

	// utworzenie hodowcw
	Breeder one(breeds1, SIZE);
	Breeder two(breeds2, SIZE);

	// wyznaczenie ras psw, ktre s hodowane przez obu hodowcw
	StringVector common_breeds = CommonBreeds(one,two);

	// wywietlenie rezultatw
	cout << "Rasy hodowcy 1:" << endl;
	show_collection<StringVector>(one.getSortedBreeds());
	cout << endl;
	
	cout << "Rasy hodowcy 2:" << endl;
	show_collection<StringVector>(two.getSortedBreeds());
	cout << endl;

	cout << "Wspolne rasy psow:" << endl;
	show_collection<StringVector>(common_breeds);

	std::cin.get();
	return 0;
}
